/* BuddyController.m created by epeyton on Thu 18-Mar-1999 */

#import "BuddyController.h"

@implementation BuddyController

- (id)init
{
    NSMutableDictionary *buddyList;
    NSMutableDictionary	*groupList;
    
    self = [super init];

    [NSBundle loadNibNamed:BUDDY_WINDOW owner:self];

    groups = [[NSMutableArray alloc] init];
    buddies = [[NSMutableArray alloc] init];

    buddyList = [[[NSApp delegate] configDict] objectForKey:BUDDIES_KEY];

    if (!buddyList) {
        buddyList = [NSMutableDictionary new];
    }

    groupList = [[[NSApp delegate] configDict] objectForKey:GROUPS_KEY];

    if (!groupList) {
        groupList = [NSMutableDictionary new];
    }

    [self createBuddiesFromDict:buddyList];
    [self createGroupsFromDict:groupList];    // Have to have the groups first


    
    return self;
}

- (void)dealloc
{
    [groups release];
    [buddies release];
    
    [super dealloc];
    return;
}

- (NSWindow *)window
{
    return window;
}    

- (void)awakeFromNib
{
    NSTabViewItem *onlineTabItem = [[NSTabViewItem alloc] initWithIdentifier:nil];
    NSTabViewItem *setupTabItem = [[NSTabViewItem alloc] initWithIdentifier:nil];

    [onlineTabItem setLabel:ONLINE_TAB_TEXT];
    [setupTabItem setLabel:SETUP_TAB_TEXT];

    [onlineTabItem setView:onlineListView];
    [setupTabItem setView:setupListView];

    [tabView addTabViewItem:onlineTabItem];
    [tabView addTabViewItem:setupTabItem];

    [self setupOnlineListView];
    [self setupSetupListView];

    [onlineListView reloadData];
    [setupListView reloadData];

    [window setFrameAutosaveName:BUDDY_WINDOW];
    [window setFrameUsingName:BUDDY_WINDOW];
    
    [window makeKeyAndOrderFront:self];
    
    return;
}

// parsing
- (void)createBuddiesFromDict:(NSDictionary *)aDict
{
    NSEnumerator *dictEnum = [aDict keyEnumerator];
    id aBuddy = nil;

    // rip through the buddy list and add the buddy
    while (aBuddy = [dictEnum nextObject]) {
        BuddyItem *aNewBuddy = [[[BuddyItem alloc] initWithBuddyName:aBuddy] autorelease];
        [aNewBuddy setService:[[aDict objectForKey:aBuddy] objectForKey:SERVICE_KEY]];
        [buddies addObject:aNewBuddy];
    }

    return;
}

- (void)createGroupsFromDict:(NSDictionary *)aDict
{
    NSEnumerator *dictEnum = [aDict keyEnumerator];
    id aGroup = nil;

    // rip through the buddy list and add the buddy
    while (aGroup = [dictEnum nextObject]) {
        GroupItem *newGroup = [[[GroupItem alloc] initWithGroupName:aGroup] autorelease];
        if ([[aDict objectForKey:aGroup] objectForKey:BUDDIES_KEY]) {
        	[newGroup setBuddyNames:[[aDict objectForKey:aGroup] objectForKey:BUDDIES_KEY]];
        }    

        [groups addObject:newGroup];
    }

    return;
}


// Ivar access
- (NSArray *)buddies { return buddies; }
- (NSArray *)groups { return groups; }

// altering the buddy list
- (void)saveBuddies
{
    return;
}

- (void)addBuddyNamed:(NSString *)name
{
    BuddyItem *aNewBuddy = [[[BuddyItem alloc] initWithBuddyName:name] autorelease];
    [aNewBuddy setService:AIM_SERVICE];
    [buddies addObject:aNewBuddy];
    
    return;
}

- (void)removeBuddyNamed:(NSString *)name
{
    [[BuddyItem buddyNamed:name] prepareForRemoval];
    [buddies removeObject:[BuddyItem buddyNamed:name]];

    return;
}

// notifications
- (void)buddyLoggedOn:(NSString *)buddyName
{
    NSEnumerator *groupEnum = [groups objectEnumerator];
    id aGroup = nil;

    while (aGroup = [groupEnum nextObject]) {
        if ([[aGroup buddyNames] containsObject:buddyName] && ![[aGroup onlineBuddyNames] containsObject:buddyName]) {
            [aGroup addToOnlineBuddyNames:buddyName];
            [[NSApp delegate] playSound:BUDDY_ARRIVED];

        }
    }    

    [onlineListView reloadData];
    [setupListView reloadData];

    return;
}

- (void)buddyLoggedOff:(NSString *)buddyName
{
    NSEnumerator *groupEnum = [groups objectEnumerator];
    id aGroup = nil;

    while (aGroup = [groupEnum nextObject]) {
        if ([[aGroup buddyNames] containsObject:buddyName] && [[aGroup onlineBuddyNames] containsObject:buddyName]) {
            [aGroup removeFromOnlineBuddyNames:buddyName];
            [[NSApp delegate] playSound:BUDDY_LEFT];

        }
    }    

    [onlineListView reloadData];
    [setupListView reloadData];

    return;
}

- (void)sendMessage:(id)sender
{
    if ([[[sender itemAtRow:[sender selectedRow]] class] isEqual:[BuddyItem class]]) {
    	[[NSApp delegate] newEmptyChatWindowForUser:[[sender itemAtRow:[sender selectedRow]] buddyName]];
    }    
    return;
}    

/* outline list view setup */
- (void)setupOnlineListView
{
    NSTableColumn *column;

    [onlineListView setAutoresizesAllColumnsToFit:NO];
    [onlineListView setIntercellSpacing:NSMakeSize(2.0,1.0)];
    [onlineListView addTableColumn:[[[NSTableColumn alloc] init] autorelease]];

    column = [[onlineListView tableColumns] objectAtIndex:0];
    [column setEditable:NO];
    [column setIdentifier:EMPTY_STRING];
    [[column dataCell] setWraps:NO];

    [onlineListView setAutoresizesAllColumnsToFit:YES];
    [onlineListView sizeToFit];
    [onlineListView tile];
    [onlineListView setOutlineTableColumn:column];
	[onlineListView setBackgroundColor:[NSColor whiteColor]];

    [onlineListView setDataSource:self];
    [onlineListView setDelegate:self];
    [onlineListView setTarget: self];
    [onlineListView setDoubleAction:@selector(sendMessage:)];
}

- (void)setupSetupListView
{
    NSTableColumn *column;

    [setupListView setAutoresizesAllColumnsToFit:NO];
    [setupListView setIntercellSpacing:NSMakeSize(2.0,1.0)];
    [setupListView addTableColumn:[[[NSTableColumn alloc] init] autorelease]];


    column = [[setupListView tableColumns] objectAtIndex:0];
    [column setEditable:NO];
    [column setIdentifier:EMPTY_STRING];
    [[column dataCell] setWraps:NO];

    [setupListView setAutoresizesAllColumnsToFit:YES];
    [setupListView sizeToFit];
    [setupListView tile];
    [setupListView setOutlineTableColumn:column];

    [setupListView setDataSource:self];
    [setupListView setDelegate:self];
    [setupListView setTarget: self];
}

// Data Source Delegation
- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
{
    id retValue = nil;
    if (!item) {
        if (outlineView == onlineListView) {
            if ([groups count]) {
                retValue = [groups objectAtIndex:index];
            }
        }
        if (outlineView == setupListView) {
            if ([groups count]) {
                retValue = [groups objectAtIndex:index];
            }
        }
    } else {
        if (outlineView == onlineListView) {
            if ([[item onlineBuddyNames] count]) {
                retValue = [BuddyItem buddyNamed:[[item onlineBuddyNames] objectAtIndex:index]];
            }
        }
        if (outlineView == setupListView) {
            if ([[item buddyNames] count]) {
                retValue = [BuddyItem buddyNamed:[[item buddyNames] objectAtIndex:index]];
            }
        }
    }
    return retValue;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
    BOOL retValue = NO;
    if ([[item class] isEqual:[GroupItem class]]) {
        retValue = YES;
    }
    return retValue;
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
    int retValue = 0;
    if (!item) {
        if (outlineView == onlineListView) {
            retValue = [groups count];
        }
        if (outlineView == setupListView) {
            retValue = [groups count];
        }
    } else if ([[item class] isEqual:[GroupItem class]]) {
        if (outlineView == onlineListView) {
            retValue = [[item onlineBuddyNames] count];
        }
        if (outlineView == setupListView) {
            retValue = [[item buddyNames] count];
        }
    }  

    return retValue;
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
    if ([[item class] isEqual:[GroupItem class]])
        return [item groupName];
    else
        return [item buddyName];
}

// Delegate messages
- (void)outlineViewItemDidCollapse:(NSNotification *)notification
{
    [onlineListView display];
    [setupListView display];
    return;
}

- (void)outlineViewItemDidExpand:(NSNotification *)notification
{
    [onlineListView display];
    [setupListView display];
    return;
}

- (void)outlineViewSelectionDidChange:(NSNotification *)notification
{
    [onlineListView display];
    [setupListView display];
    return;
}

@end
