/* ChatWindow.m created by epeyton on Tue 16-Mar-1999 */

#import "ChatWindow.h"

@implementation ChatWindow

- (id)init
{
    self = [super init];

    [NSBundle loadNibNamed:@"ChatWindow" owner:self];
    [window makeKeyAndOrderFront:self];
    [displayText setRichText:YES];
    [sendText setRichText:YES];
    [displayText setBackgroundColor:[NSColor whiteColor]];

    return self;
}    

/*
    Action method send:
    Sends the current message buffer to the select user(s) for this chat window
*/
- (void)send:(id)sender
{
    // ask the communication controller to send this message
    // if it comes back ok then add this sent message to the display
    // and clear out the send text field
    if (!user) {
        [self setUser:[userCombo stringValue]];
    }    

    if (!user || ![[sendText string] length]) return;
    
    [[[NSApp delegate] communicationController] sendIMToUser:user message:[NSString stringWithString:[sendText string]]];
    [self addMessageToDisplay:[sendText string] fromUser:[[NSApp delegate] myScreenName]];
    [sendText setString:EMPTY_STRING];
    [[NSApp delegate] playSound:MSG_SEND];

}

/*
    Ivar access and setting
*/
- (void)setUser:(NSString *)aUserName
{
    if (user) [user autorelease];
    user = [aUserName retain];

    [window setTitle:[NSString stringWithFormat:@"%@ - Instant Message", aUserName]];
    [userCombo setEnabled:NO];
    [userCombo setFrame:NSMakeRect(0,0,0,0)];
    [[NSApp delegate] registerChatWindow:self];

    // also tell the main controller that a chat window exists with that user already

    return;
}

- (NSString *)user
{
    return user;
}    

/*
    This method adds a new message to the display for reading
*/
- (void)addMessageToDisplay:(NSString *)aMessage fromUser:(NSString *)userName
{
    int htmlExists = 1;
    // add a new line
    // add the message senders name
    // if displayTime put it in
    // then add the message
    // redisplay
    NSMutableString *newString = [NSMutableString stringWithString:[displayText string]];
    NSMutableString *messageString = [NSMutableString stringWithString:aMessage];
    if ([newString length]) {
        [newString appendString:@"\n"];
    }    

    [newString appendString:userName];
    [newString appendString:@": "];

    // Strip html formatting from the string
    while (htmlExists) {
        NSRange bracketRange1 = [messageString rangeOfString:@"<"];
        NSRange bracketRange2 = [messageString rangeOfString:@">"];

        if ((bracketRange1.length > 0) && (bracketRange2.length > 0)) {
            NSRange aDeadRange;
            
            aDeadRange.location = bracketRange1.location;
            aDeadRange.length = bracketRange2.location - bracketRange1.location + bracketRange2.length;
            [messageString replaceCharactersInRange: aDeadRange withString:@""];
            
        } else {
            htmlExists = 0;
        }
    }    

    [newString appendString:messageString];

    [displayText setString:newString];

    [displayText scrollRangeToVisible:NSMakeRange([[displayText textStorage] length], 0)];
    
}

- (void)display
{
    [window display];
    return;
}

- (void)windowWillClose:(id)sender
{
    // tell the main controller to "unregister" this window for this user
    [[NSApp delegate] unregisterChatWindow:self];
    return;
}

@end
