#import "LoginController.h"

@implementation LoginController

- (void)awakeFromNib
{
    [self setupWithAutoLogin:[[NSUserDefaults standardUserDefaults] integerForKey:AUTO_LOGIN]];

    [window setFrameAutosaveName:LOGIN_WINDOW];
    [window setFrameUsingName:LOGIN_WINDOW];

    return;
}    

- (void)setupWithAutoLogin:(BOOL)autologin
{
    id screenNameString = [[NSUserDefaults standardUserDefaults] objectForKey:SCREEN_NAME_DEFAULT_TEXT];
    id passwordString = [[NSUserDefaults standardUserDefaults] objectForKey:PASSWORD_DEFAULT_TEXT];

    
    if (!autologin) {

        if (screenNameString) {
            [screenNameText setStringValue:screenNameString];
        } else {
            [screenNameText setStringValue:EMPTY_STRING];
        }

        if (passwordString && [[NSUserDefaults standardUserDefaults] integerForKey:SAVE_PASSWORD] == 1) {
            [passwordText setStringValue:passwordString];
            [autoLoginCheck setEnabled:YES];
        } else {
            [passwordText setStringValue:EMPTY_STRING];
            [autoLoginCheck setEnabled:NO];
       }

        [messageString setStringValue:EMPTY_STRING];
        [versionString setStringValue:CURRENT_VERSION_STRING];

        [savePasswordCheck setIntValue:[[NSUserDefaults standardUserDefaults] integerForKey:SAVE_PASSWORD]];
        [autoLoginCheck setIntValue:[[NSUserDefaults standardUserDefaults] integerForKey:AUTO_LOGIN]];
        
    } else {
        // free the window and login ...
        BOOL ret = [self loginForUser:screenNameString password:passwordString];
        if (ret) {
            [window close];
            [window release];
        } else {
            // something bad happened get rid of the autologin and setup the panel again
            [self setupWithAutoLogin:NO];
        }    
    }        
    return;
}    

- (void)changePreference:(id)sender
{
    if ([savePasswordCheck intValue]) {
        [[NSUserDefaults standardUserDefaults] setObject:[passwordText stringValue] forKey:PASSWORD_DEFAULT_TEXT];
        // turn on the auto-login feature
        [autoLoginCheck setEnabled:YES];
    } else {
        // remove any default text that may be around
        [[NSUserDefaults standardUserDefaults] setObject:EMPTY_STRING forKey:PASSWORD_DEFAULT_TEXT];
        [autoLoginCheck setEnabled:NO];
        [autoLoginCheck setIntValue:0];
        [[NSUserDefaults standardUserDefaults] setInteger:0 forKey:AUTO_LOGIN];
    }
    
    [[NSUserDefaults standardUserDefaults] setInteger:[savePasswordCheck intValue] forKey:SAVE_PASSWORD];
    

    [[NSUserDefaults standardUserDefaults] setInteger:[autoLoginCheck intValue] forKey:AUTO_LOGIN];

    [[NSUserDefaults standardUserDefaults] setObject:[screenNameText stringValue] forKey:SCREEN_NAME_DEFAULT_TEXT];

    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (void)help:(id)sender
{
}

- (void)login:(id)sender
{
    [self changePreference:self];
    
    // Need to check for lengths and refusae of either too short
    if ([[passwordText stringValue] length] < 1) {
        [messageString setStringValue:NO_PASSWORD_FAILURE];
        return;
    }
    if ([[screenNameText stringValue] length] < 1) {
        [messageString setStringValue:NO_SCREENNAME_FAILURE];
        return;
    }

        
    if ([self loginForUser:[screenNameText stringValue] password:[passwordText stringValue]]) {
        [window close];
    }    
}

- (BOOL)loginForUser:(NSString *)userName password:(NSString *)password
{
    BOOL returnValue = NO;
    int ret = aim_login([userName cString], [password cString]);
    if (ret) {
        if (ret == LOGIN_FAILURE_TOO_SOON) {
            [messageString setStringValue:TOO_SOON_FAILURE];
        }
        [messageString setStringValue:LOGIN_FAILED];
    } else {
        char *buddies = NULL;
        /* this is the new profile */
        char profile[] = "No profile specified.";
        //close the login window and create a main controller
        aim_send_login_phase3c_hi(buddies, profile);
        aim_send_login_phase4_a();
        [[[NSApp delegate] communicationController] initializeRunLoopCheck];
        [[NSApp delegate] setMyScreenName:userName];
        returnValue = YES;
        
    }
    return returnValue;
}    

- (void)setup:(id)sender
{
}

@end
